
#ifndef CONFIG_H_
#define CONFIG_H_

#include <map>
#include <vector>

using namespace std;

    /**
     * Diese Struct wird zur Sortierung der Map der ConfigObjekts gebraucht
     */

    struct ltstr
    {
        bool operator()(const string s1, const string s2) const
        {
            return s1.compare(s2) < 0;
        }
    };

    /**
     * Die Klasse Config ist zum ablegen und laden von generischen Konfigurationsparametern
     * gedacht.
     *
     * Dabei kann in einem Config Objekt ein Werte unter einer Sektion und einem Schlüssel
     * ablegen. Sektion, Schlüssel und Wert sind Strings.
     *
     * Die Config Klasse kann sich aus einem Stream "laden" und in einen Stream
     * serialisieren. Das Format ist dabei and das INI Format angelehnt.
     *
     * Eine Beispiel Konfigurationsdatei künnte wie folgt aussehen:
     * @code
     * # Kommentar
     * [section] #Die Erste Sektion
     * key=value #Ein Wert
     * #key=value Auskommentiert
     * [andere section] #Zweite Sektion
     * foo=bar
     * @endcode
     */

    class Config
    {

        public:
            /**
             * Konstruktur
             */
            Config();

            /**
             * Destruktor
             */
            ~Config();

            /**
             * Lädt die Config aus einem Stream
             */
            istream& load(istream &);

            /**
             * Speichert die Config in einen Stream
             */
            ostream& save(ostream &) const;

            /**
             * Diese Funktion kann benutzt werden um Werte zu (Sektion, Schlüssel)
             * Paaren aus dem Config Objekt zu holen.
             *
             * @param section Die Sektion in der der gesuchte Wert liegt
             * @param key Der Schlüssel des gesuchten Wertes
             * @return Den Wert zu (Sektion, Schlüssel) 
             */
            string getStringValue(string section, string key) const;

            /**
             * Diese Funktion kann benutzt werden um Werte zu (Sektion, Schlüssel)
             * Paaren aus dem Config Objekt zu holen.
             *
             * @param section Die Sektion in der der gesuchte Wert liegt
             * @param key Der Schlüssel des gesuchten Wertes
             * @return Den Wert zu (Sektion, Schlüssel) 
             */
            int getIntValue(string section, string key) const;

            /**
             * Diese Funktion kann benutzt werden um Werte zu (Sektion, Schlüssel)
             * Paaren aus dem Config Objekt zu holen.
             *
             * @param section Die Sektion in der der gesuchte Wert liegt
             * @param key Der Schlüssel des gesuchten Wertes
             * @return Den Wert zu (Sektion, Schlüssel) 
             */
            float getFloatValue(string section, string key) const;

            /**
             * Diese Funktion kann benutzt werden um Werte zu (Sektion, Schlüssel)
             * Paaren aus dem Config Objekt zu holen.
             *
             * @param section Die Sektion in der der gesuchte Wert liegt
             * @param key Der Schlüssel des gesuchten Wertes
             * @return Den Wert zu (Sektion, Schlüssel) 
             */
            double getDoubleValue(string section, string key) const;

            /**
             * Diese Funktion setzt einen Wert zu einem (Sektion, Schlüssel) Paar.
             * Dabei werden Sektion und oder Schüssel angelegt, wenn sie nicht
             * existieren.
             * Existsiert das Paar schon so wird der alte Wert überschrieben.
             *
             * @param section Die Sektion des Wertes der gesetzt werden soll.
             * @param key Der Schlüssel des Wertes der gesetzt werden soll.
             * @param value Der Wert auf den das (Sektion, Schlüssel) Paar gesetzt werden soll.
             */
            void setValue(string section, string key, string value);

            /**
             * Diese Funktion kann ein (Sektion, Schlüssel) Paar aus dem Config Objekt
             * löschen.
             *
             * Ist eine Sektion nach dem Löschen ohne Key, so wird diese Sektion auch gelöscht.
             * @param section Die Sektion aus der ein Key gelöscht werden soll.
             * @param key Der Schlüssel der gelöscht werden soll.
             */
            void removeValue(string section, string key);

            /**
             * Diese Funktion liefert einen Vektor der Sektionsnamen.
             *
             * @return Ein Vektor der Sektionsnamen.
             */
            vector<string> getSectionNames();

            /**
             * Diese Funktion liefert einen Vektor der Schlüsselnamen einer Sektion.
	         *
             * @return Ein Vektor der Schlüsselnamen dieser Sektion.
             */
            vector<string> getKeyNames(string section);

        private :
            typedef map<string, string, ltstr> KeyValueMap;
            typedef map<string, KeyValueMap *, ltstr > SectionKeyValueMap;

            /** Die Interne Assoziative Datenstruktur */
            SectionKeyValueMap m_data;

    };  //of class


#endif /*CONFIG_H_*/
